@@ -60,11 +60,27 @@ class Optimizer:
 
         :param np.array xk:
             Array with parameters for optimization
+
         :return: float result:
             A scalar (float/ 1d) value for the optimization framework.
         """
         raise NotImplementedError(f'{self.__class__.__name__}.obj function is not defined')
 
+    @abstractmethod
+    def mp_obj(self, x, *args):
+        """
+        Objective function for Multiprocessing.
+
+        :param np.array x:
+            Array with parameters for optimization.
+            Shape of the array is (number_of_evaluations x number_of_variables).
+            For instance, optimizating 10 variables and evaluating
+            900 objectives in parallel, the shape would be 900 x 10.
+        :param int n_cpu:
+            Number of logical Processors to run optimization on.
+        """
+        raise NotImplementedError(f'{self.__class__.__name__}.obj function is not defined')
+
     @property
     def supported_frameworks(self):
         """
@@ -97,7 +113,7 @@ class Optimizer:
         """Set the boundaries to the optimization variables"""
         self._bounds = bounds
 
-    def optimize(self, framework, method=None, **kwargs):
+    def optimize(self, framework, method=None, n_cpu=1, **kwargs):
         """
         Perform the optimization based on the given method and framework.
 
@@ -118,6 +134,10 @@ class Optimizer:
             strategy.
             For the pymoo function, method is equal to the
             algorithm.
+        :param int n_cpu:
+            Number of parallel processes used for the evaluation.
+            Ignored if the framework-method combination does not
+            support multi-processing.
 
         Keyword arguments:
             Depending on the framework an method you use, you can fine-tune the
@@ -136,7 +156,7 @@ class Optimizer:
             raise ValueError(f"{framework} requires a method, but None is "
                              f"provided. Please choose one.")
         # Perform minimization
-        res = minimize_func(method, **kwargs)
+        res = minimize_func(method=method, n_cpu=n_cpu, **kwargs)
         return res
 
     def _choose_framework(self, framework):
@@ -161,7 +181,7 @@ class Optimizer:
             return self._pymoo, True
         raise TypeError(f"Given framework {framework} is currently not supported.")
 
-    def _scipy_minimize(self, method, **kwargs):
+    def _scipy_minimize(self, method, n_cpu=1, **kwargs):
         """
         Possible kwargs for the scipy minimize function with default values:
 
@@ -202,7 +222,7 @@ class Optimizer:
             # pylint: disable=inconsistent-return-statements
             self._handle_error(error)
 
-    def _dlib_minimize(self, _, **kwargs):
+    def _dlib_minimize(self, method=None, n_cpu=1, **kwargs):
         """
         Possible kwargs for the dlib minimize function with default values:
 
@@ -251,7 +271,7 @@ class Optimizer:
             # pylint: disable=inconsistent-return-statements
             self._handle_error(error)
 
-    def _scipy_differential_evolution(self, method="best1bin", **kwargs):
+    def _scipy_differential_evolution(self, method="best1bin", n_cpu=1, **kwargs):
         """
         Possible kwargs for the dlib minimize function with default values:
 
@@ -297,7 +317,7 @@ class Optimizer:
             # pylint: disable=inconsistent-return-statements
             self._handle_error(error)
 
-    def _pymoo(self, method="NSGA2", **kwargs):
+    def _pymoo(self, method="NSGA2", n_cpu=1, **kwargs):
         """
         Possible kwargs for the dlib minimize function with default values:
 
@@ -312,11 +332,11 @@ class Optimizer:
         copy_termination=False
         """
         default_kwargs = self.get_default_config(framework="pymoo")
-        default_kwargs.update(kwargs)
+
         try:
             from pymoo.optimize import minimize
             from pymoo.problems.single import Problem
-            from pymoo.factory import get_algorithm
+            from pymoo.factory import get_algorithm, get_sampling, get_mutation, get_crossover, get_selection
         except ImportError as error:
             raise ImportError("Please install pymoo to use this function.") from error
 
@@ -334,7 +354,10 @@ class Optimizer:
                                  )
 
             def _evaluate(self, x, out, *args, **kwargs):
-                out["F"] = np.array([self.ebcpy_class.obj(xk=_x, *args) for _x in x])
+                if n_cpu > 1:
+                    out["F"] = self.ebcpy_class.mp_obj(x, n_cpu, *args)
+                else:
+                    out["F"] = np.array([self.ebcpy_class.obj(xk=_x, *args) for _x in x])
 
         try:
             if self.bounds is None:
@@ -351,8 +374,28 @@ class Optimizer:
             copy_termination = default_kwargs.pop("copy_termination")
 
             # Init algorithm
-            algorithm = get_algorithm(name=method.lower(),
-                                      **default_kwargs)
+            if method.lower() == "ga":
+                from pymoo.algorithms.soo.nonconvex.ga import GA
+                # GA:
+                pop_size = kwargs["pop_size"]
+                sampling = get_sampling(name=kwargs["sampling"])
+                selection = get_selection(name=kwargs["selection"])
+                crossover = get_crossover(name=kwargs["crossover"])
+                mutation = get_mutation(name=kwargs["mutation"])
+                eliminate_duplicates = kwargs["eliminate_duplicates"]
+                n_offsprings = kwargs["n_offsprings"]
+                algorithm = GA(pop_size=pop_size,
+                               sampling=sampling,
+                               selection=selection,
+                               crossover=crossover,
+                               mutation=mutation,
+                               eliminate_duplicates=eliminate_duplicates,
+                               n_offsprings=n_offsprings
+                               )
+            else:
+                default_kwargs.update(kwargs)
+                algorithm = get_algorithm(name=method.lower(),
+                                          **default_kwargs)
 
             res = minimize(
                 problem=EBCPYProblem(ebcpy_class=self),
