@@ -5,6 +5,8 @@ Parameters can easily be updated, and the initialization-process is
 much more user-friendly than the provided APIs by Dymola or fmpy.
 """
 import logging
+import pathlib
+import warnings
 import os
 import itertools
 from typing import Dict, Union, TypeVar, Any, List
@@ -13,6 +15,7 @@ import multiprocessing as mp
 from pydantic import BaseModel, Field, validator
 import numpy as np
 from ebcpy.utils import setup_logger
+from ebcpy.utils.reproduction import save_reproduction_archive
 
 
 class Variable(BaseModel):
@@ -20,24 +23,59 @@ class Variable(BaseModel):
     Data-Class to store relevant information for a
     simulation variable (input, parameter, output or local/state).
     """
-    value: Any = Field(
-        description="Default variable value"
-    )
-    max: Union[float, int] = Field(
-        default=np.inf,
-        title='max',
-        description='Maximal value (upper bound) of the variables value'
-    )
-    min: Union[float, int] = Field(
-        default=-np.inf,
-        title='min',
-        description='Minimal value (lower bound) of the variables value'
-    )
     type: Any = Field(
         default=None,
         title='type',
         description='Type of the variable'
     )
+    value: Any = Field(
+        description="Default variable value"
+    )
+    max: Any = Field(
+        default=None,
+        title='max',
+        description='Maximal value (upper bound) of the variables value. '
+                    'Only for ints and floats variables.'
+    )
+    min: Any = Field(
+        default=None,
+        title='min',
+        description='Minimal value (lower bound) of the variables value. '
+                    'Only for ints and floats variables.'
+    )
+
+    @validator("value")
+    def check_value_type(cls, value, values):
+        """Check if the given value has correct type"""
+        _type = values["type"]
+        if _type is None:
+            return value   # No type -> no conversion
+        if value is None:
+            return value  # Setting None is allowed.
+        if not isinstance(value, _type):
+            return _type(value)
+        return value
+
+    @validator('max', 'min', always=True)
+    def check_value(cls, value, values, field):
+        """Check if the given bounds are correct."""
+        # Check if the variable type even allows for min/max bounds
+        _type = values["type"]
+        if _type is None:
+            return value   # No type -> no conversion
+        if _type not in (float, int, bool):
+            if value is not None:
+                warnings.warn(
+                    "Setting a min/max for variables "
+                    f"of type {_type} is not supported."
+                )
+            return None
+        if value is not None:
+            return _type(value)
+        if field.name == "min":
+            return -np.inf if _type != bool else False
+        # else it is max
+        return np.inf if _type != bool else True
 
 
 class SimulationSetup(BaseModel):
@@ -162,7 +200,6 @@ class SimulationAPI:
         self_dict = self.__dict__.copy()
         for item in self._items_to_drop:
             del self_dict[item]
-        #return deepcopy(self_dict)
         return self_dict
 
     def __setstate__(self, state):
@@ -352,6 +389,12 @@ class SimulationAPI:
         # Empty all variables again.
         if self.worker_idx:
             return
+        self._update_model_variables()
+
+    def _update_model_variables(self):
+        """
+        Function to empty all variables and update them again
+        """
         self.outputs = {}
         self.parameters = {}
         self.states = {}
@@ -442,3 +485,34 @@ class SimulationAPI:
     def get_simulation_setup_fields(cls):
         """Return all fields in the chosen SimulationSetup class."""
         return list(cls._sim_setup_class.__fields__.keys())
+
+    def save_for_reproduction(self,
+                              title: str,
+                              path: pathlib.Path = None,
+                              files: list = None,
+                              **kwargs):
+        """
+        Save the settings of the SimulationAPI in order to
+        reproduce the settings of the used simulation.
+
+        Should be extended by child-classes to allow custom
+        saving.
+
+        :param str title:
+            Title of the study
+        :param pathlib.Path path:
+            Where to store the .zip file. If not given, self.cd is used.
+        :param list files:
+            List of files to save along the standard ones.
+            Examples would be plots, tables etc.
+        :param dict kwargs:
+            All keyword arguments except files and path of the function
+            save_reproduction_archive
+        """
+        if path is None:
+            path = self.cd
+        return save_reproduction_archive(
+            title=title,
+            path=path,
+            files=files
+        )
