@@ -7,6 +7,7 @@ import shutil
 import pathlib
 import warnings
 import atexit
+import json
 from typing import Union, List
 from pydantic import Field
 import pandas as pd
@@ -112,22 +113,24 @@ class DymolaAPI(SimulationAPI):
 
     """
     _sim_setup_class: SimulationSetupClass = DymolaSimulationSetup
-    _dymola_instances: dict = {}
     _items_to_drop = ["pool", "dymola", "_dummy_dymola_instance"]
+    dymola = None
     # Default simulation setup
-    _supported_kwargs = ["show_window",
-                         "modify_structural_parameters",
-                         "dymola_path",
-                         "equidistant_output",
-                         "n_restart",
-                         "debug",
-                         "mos_script_pre",
-                         "mos_script_post",
-                         "dymola_version"]
+    _supported_kwargs = [
+        "show_window",
+        "modify_structural_parameters",
+        "dymola_path",
+        "equidistant_output",
+        "n_restart",
+        "debug",
+        "mos_script_pre",
+        "mos_script_post",
+        "dymola_version"
+    ]
 
     def __init__(self, cd, model_name, packages=None, **kwargs):
         """Instantiate class objects."""
-
+        self.dymola = None  # Avoid key-error in get-state. Instance attribute needs to be there.
         # Update kwargs with regard to what kwargs are supported.
         self.extract_variables = kwargs.pop("extract_variables", True)
         self.fully_initialized = False
@@ -156,8 +159,6 @@ class DymolaAPI(SimulationAPI):
             self.mos_script_pre = self._make_modelica_normpath(self.mos_script_pre)
         if self.mos_script_post is not None:
             self.mos_script_post = self._make_modelica_normpath(self.mos_script_post)
-        # Set empty dymola attribute
-        self.dymola = None
 
         super().__init__(cd=cd,
                          model_name=model_name,
@@ -264,6 +265,10 @@ class DymolaAPI(SimulationAPI):
 
         Additional settings:
 
+        :keyword List[str] model_names:
+            List of Dymola model-names to simulate. Should be either the size
+            of parameters or parameters needs to be sized 1.
+            Keep in mind that different models may use different parameters!
         :keyword Boolean show_eventlog:
             Default False. True to show evenlog of simulation (advanced)
         :keyword Boolean squeeze:
@@ -301,6 +306,15 @@ class DymolaAPI(SimulationAPI):
             # the super method.
             if not isinstance(_struc_params[0], list):
                 kwargs["structural_parameters"] = [_struc_params]
+        if "model_names" in kwargs:
+            model_names = kwargs["model_names"]
+            if not isinstance(model_names, list):
+                raise TypeError("model_names needs to be a list.")
+            if isinstance(parameters, dict):
+                # Make an array of parameters to enable correct use of super function.
+                parameters = [parameters] * len(model_names)
+            if parameters is None:
+                parameters = [{}] * len(model_names)
         return super().simulate(parameters=parameters, return_option=return_option, **kwargs)
 
     def _single_simulation(self, kwargs):
@@ -310,6 +324,7 @@ class DymolaAPI(SimulationAPI):
         result_file_name = kwargs.get("result_file_name", 'resultFile')
         parameters = kwargs.get("parameters")
         return_option = kwargs.get("return_option")
+        model_names = kwargs.get("model_names")
         inputs = kwargs.get("inputs", None)
         fail_on_error = kwargs.get("fail_on_error", True)
         structural_parameters = kwargs.get("structural_parameters", [])
@@ -317,21 +332,35 @@ class DymolaAPI(SimulationAPI):
         # Handle multiprocessing
         if self.use_mp:
             idx_worker = self.worker_idx
-            if idx_worker not in self._dymola_instances:
+            if self.dymola is None:
                 self._setup_dymola_interface(use_mp=True)
-            dymola = self._dymola_instances[idx_worker]
-        else:
-            dymola = self.dymola
 
         # Handle eventlog
         if show_eventlog:
-            dymola.experimentSetupOutput(events=True)
-            dymola.ExecuteCommand("Advanced.Debug.LogEvents = true")
-            dymola.ExecuteCommand("Advanced.Debug.LogEventsInitialization = true")
+            self.dymola.experimentSetupOutput(events=True)
+            self.dymola.ExecuteCommand("Advanced.Debug.LogEvents = true")
+            self.dymola.ExecuteCommand("Advanced.Debug.LogEventsInitialization = true")
 
         # Restart Dymola after n_restart iterations
         self._check_restart()
 
+        # Handle custom model_names
+        if model_names is not None:
+            # Custom model_name setting
+            _res_names = self.result_names.copy()
+            self._model_name = model_names
+            self._update_model_variables()
+            if _res_names != self.result_names:
+                self.logger.info(
+                    "Result names changed due to setting the new model. "
+                    "If you do not expect custom result names, ignore this warning."
+                    "If you do expect them, please raise an issue to add the "
+                    "option when using the model_names keyword.")
+                self.logger.info(
+                    "Difference: %s",
+                    " ,".join(list(set(_res_names).difference(self.result_names)))
+                )
+
         # Handle parameters:
         if parameters is None:
             parameters = {}
@@ -358,13 +387,13 @@ class DymolaAPI(SimulationAPI):
                 _res_names = self.result_names.copy()
                 self.model_name = model_name
                 self.result_names = _res_names  # Restore previous result names
-            self.logger.warning(
-                "Warning: Currently, the model is re-translating "
-                "for each simulation. You should add to your Modelica "
-                "parameters \"annotation(Evaluate=false)\".\n "
-                "Check for these parameters: %s",
-                ', '.join(set(parameters.keys()).difference(parameters_new.keys()))
-            )
+                self.logger.warning(
+                    "Warning: Currently, the model is re-translating "
+                    "for each simulation. You should add to your Modelica "
+                    "parameters \"annotation(Evaluate=false)\".\n "
+                    "Check for these parameters: %s",
+                    ', '.join(set(parameters.keys()).difference(parameters_new.keys()))
+                )
             parameters = parameters_new
             # Check again
             unsupported_parameters = self.check_unsupported_variables(
@@ -381,6 +410,7 @@ class DymolaAPI(SimulationAPI):
             raise TypeError("Dymola only accepts float values. "
                             "Could bot automatically convert the given "
                             "parameter values to float.") from err
+
         # Handle inputs
         if inputs is not None:
             # Unpack additional kwargs
@@ -409,7 +439,7 @@ class DymolaAPI(SimulationAPI):
                                "names for option return_type='savepath'. "
                                "To use this option, delete unsupported "
                                "parameters from your setup.")
-            res = dymola.simulateExtendedModel(
+            res = self.dymola.simulateExtendedModel(
                 self.model_name,
                 startTime=self.sim_setup.start_time,
                 stopTime=self.sim_setup.stop_time,
@@ -455,7 +485,7 @@ class DymolaAPI(SimulationAPI):
                     "numberOfIntervals or a value for output_interval "
                     "which can be converted to numberOfIntervals.")
 
-            res = dymola.simulateMultiResultsModel(
+            res = self.dymola.simulateMultiResultsModel(
                 self.model_name,
                 startTime=self.sim_setup.start_time,
                 stopTime=self.sim_setup.stop_time,
@@ -471,7 +501,7 @@ class DymolaAPI(SimulationAPI):
         if not res[0]:
             self.logger.error("Simulation failed!")
             self.logger.error("The last error log from Dymola:")
-            log = dymola.getLastErrorLog()
+            log = self.dymola.getLastErrorLog()
             # Only print first part as output is sometimes to verbose.
             self.logger.error(log[:10000])
             dslog_path = os.path.join(self.cd, 'dslog.txt')
@@ -493,9 +523,9 @@ class DymolaAPI(SimulationAPI):
             _save_name_dsres = f"{result_file_name}.mat"
             savepath = kwargs.pop("savepath", None)
             # Get the cd of the current dymola instance
-            dymola.cd()
+            self.dymola.cd()
             # Get the value and convert it to a 100 % fitting str-path
-            dymola_cd = str(pathlib.Path(dymola.getLastErrorLog().replace("\n", "")))
+            dymola_cd = str(pathlib.Path(self.dymola.getLastErrorLog().replace("\n", "")))
             if savepath is None or str(savepath) == dymola_cd:
                 return os.path.join(dymola_cd, _save_name_dsres)
             os.makedirs(savepath, exist_ok=True)
@@ -645,27 +675,25 @@ class DymolaAPI(SimulationAPI):
         super().close()
         # Always close main instance
         self._single_close(dymola=self.dymola)
-        self.dymola = None
 
     def _close_multiprocessing(self, _):
-        wrk_idx = self.worker_idx
-        if wrk_idx in self._dymola_instances:
-            self._single_close(dymola=self._dymola_instances.pop(wrk_idx))
+        self._single_close()
+        DymolaAPI.dymola = None
 
     def _single_close(self, **kwargs):
         """Closes a single dymola instance"""
-        dymola = kwargs["dymola"]
-        if dymola is None:
+        if self.dymola is None:
             return  # Already closed prior
         # Execute the mos-script if given:
         if self.mos_script_post is not None:
             self.logger.info("Executing given mos_script_post "
                              "prior to closing.")
-            dymola.RunScript(self.mos_script_post)
-            self.logger.info("Output of mos_script_post: %s", dymola.getLastErrorLog())
+            self.dymola.RunScript(self.mos_script_post)
+            self.logger.info("Output of mos_script_post: %s", self.dymola.getLastErrorLog())
         self.logger.info('Closing Dymola')
-        dymola.close()
+        self.dymola.close()
         self.logger.info('Successfully closed Dymola')
+        self.dymola = None
 
     def _close_dummy(self):
         """
@@ -744,8 +772,8 @@ class DymolaAPI(SimulationAPI):
             warnings.warn("You have no licence to use Dymola. "
                           "Hence you can only simulate models with 8 or less equations.")
         if use_mp:
-            self._dymola_instances[self.worker_idx] = dymola
-            return True
+            DymolaAPI.dymola = dymola
+            return None
         return dymola
 
     def _open_dymola_interface(self):
@@ -771,8 +799,9 @@ class DymolaAPI(SimulationAPI):
         :return: dict config:
             Dictionary with keys to re-init this class.
         """
-        config = {"cd": self.cd,
-                  "packages": self.packages,
+        # Convert Path to str to enable json-dumping
+        config = {"cd": str(self.cd),
+                  "packages": [str(pack) for pack in self.packages],
                   "model_name": self.model_name,
                   "type": "DymolaAPI",
                   }
@@ -782,6 +811,147 @@ class DymolaAPI(SimulationAPI):
 
         return config
 
+    def get_packages(self):
+        """
+        Get the currently loaded packages of Dymola
+        """
+        packages = self.dymola.ExecuteCommand(
+            'ModelManagement.Structure.AST.Misc.ClassesInPackage("")'
+        )
+        valid_packages = []
+        for pack in packages:
+            current_package = f"modelica://{pack}/package.order"
+            pack_path = self.dymola.ExecuteCommand(
+                f'Modelica.Utilities.Files.loadResource("{current_package}")'
+            )
+            if not isinstance(pack_path, str):
+                self.logger.error("Could not load model resource for package %s", pack)
+            if os.path.isfile(pack_path):
+                valid_packages.append(pathlib.Path(pack_path).parent)
+        return valid_packages
+
+    def save_for_reproduction(
+            self,
+            title: str,
+            path: pathlib.Path = None,
+            files: list = None,
+            save_total_model: bool = True,
+            export_fmu: bool = True
+    ):
+        """
+        Additionally to the basic reproduction, add info
+        for Dymola packages.
+
+        Content which is saved:
+        - DymolaAPI configuration
+        - Information on Dymola: Version, flags
+        - All loaded packages
+        - Total model, if save_total_model = True
+        - FMU, if export_fmu = True
+
+        :param bool save_total_model:
+            True to save the total model
+        :param bool export_fmu:
+            True to export the FMU of the current model.
+        """
+        # Local import to require git-package only when called
+        from ebcpy.utils.reproduction import ReproductionFile, CopyFile, get_git_information
+
+        if files is None:
+            files = []
+        # DymolaAPI Info:
+        files.append(ReproductionFile(
+            filename="Dymola/DymolaAPI_config.json",
+            content=json.dumps(self.to_dict(), indent=2)
+        ))
+        # Dymola info:
+        self.dymola.ExecuteCommand("list();")
+        _flags = self.dymola.getLastErrorLog()
+        dymola_info = [
+            self.dymola.ExecuteCommand("DymolaVersion()"),
+            str(self.dymola.ExecuteCommand("DymolaVersionNumber()")),
+            "\n\n"
+        ]
+        files.append(ReproductionFile(
+            filename="Dymola/DymolaInfo.txt",
+            content="\n".join(dymola_info) + _flags
+        ))
+
+        # Packages
+        packages = self.get_packages()
+        package_infos = []
+        for pack_path in packages:
+
+            for pack_dir_parent in [pack_path] + list(pack_path.parents):
+                repo_info = get_git_information(
+                    path=pack_dir_parent,
+                    zip_folder_path="Dymola"
+                )
+                if not repo_info:
+                    continue
+
+                files.extend(repo_info.pop("difference_files"))
+                pack_path = str(pack_path) + "; " + "; ".join([f"{key}: {value}" for key, value in repo_info.items()])
+                break
+            package_infos.append(str(pack_path))
+        files.append(ReproductionFile(
+            filename="Dymola/Modelica_packages.txt",
+            content="\n".join(package_infos)
+        ))
+        # Total model
+        if save_total_model:
+            _total_model_name = f"Dymola/{self.model_name.replace('.', '_')}_total.mo"
+            _total_model = pathlib.Path(self.cd).joinpath(_total_model_name)
+            os.makedirs(_total_model.parent, exist_ok=True)  # Create to ensure model can be saved.
+            res = self.dymola.saveTotalModel(
+                fileName=str(_total_model),
+                modelName=self.model_name
+            )
+            if res:
+                files.append(ReproductionFile(
+                    filename=_total_model_name,
+                    content=_total_model.read_text()
+                ))
+                os.remove(_total_model)
+            else:
+                self.logger.error("Could not save total model: %s",
+                                  self.dymola.getLastErrorLog())
+        # FMU
+        if export_fmu:
+            _fmu_path = self._save_to_fmu(fail_on_error=False)
+            if _fmu_path is not None:
+                files.append(CopyFile(
+                    sourcepath=_fmu_path,
+                    filename="Dymola/" + _fmu_path.name,
+                    remove=True
+                ))
+
+        return super().save_for_reproduction(
+            title=title,
+            path=path,
+            files=files
+        )
+
+    def _save_to_fmu(self, fail_on_error):
+        """Save model as an FMU"""
+        res = self.dymola.translateModelFMU(
+            modelToOpen=self.model_name,
+            storeResult=False,
+            modelName='',
+            fmiVersion='2',
+            fmiType='all',
+            includeSource=False,
+            includeImage=0
+        )
+        if not res:
+            msg = "Could not export fmu: %s" % self.dymola.getLastErrorLog()
+            self.logger.error(msg)
+            if fail_on_error:
+                raise Exception(msg)
+        else:
+            path = pathlib.Path(self.cd).joinpath(res + ".fmu")
+            return path
+
     @staticmethod
     def _make_modelica_normpath(path):
         """
@@ -981,7 +1151,7 @@ class DymolaAPI(SimulationAPI):
         if self.sim_counter == self.n_restart:
             self.logger.info("Closing and restarting Dymola to free memory")
             self.close()
-            self.dymola = self._setup_dymola_interface(use_mp=False)
+            self._dummy_dymola_instance = self._setup_dymola_interface(use_mp=False)
             self.sim_counter = 1
         else:
             self.sim_counter += 1
